var _excluded = ["min", "max", "list", "value", "contentProps", "TabsProps", "DatePickerComboProps", "onError", "onChange"];

function _extends() { _extends = Object.assign ? Object.assign.bind() : function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

import React, { forwardRef, useRef, useEffect, useImperativeHandle } from 'react';
import Tabs from "./../../Tabs";
import locale from "./../../locale";
import InputDate from "./../Combo";
import DateComboUtils from "./../Combo/Utils";
import DateModalUtils from "./../Modal/Utils"; // 日期类型选择控件: 年月日季

var Types = /*#__PURE__*/forwardRef(function (_ref, ref) {
  var min = _ref.min,
      max = _ref.max,
      _ref$list = _ref.list,
      list = _ref$list === void 0 ? [{
    type: 'date',
    id: 'date',
    name: locale('日', 'datetype_unit_date')
  }, {
    type: 'month',
    id: 'month',
    name: locale('月', 'datetype_unit_month')
  }, {
    type: 'quarter',
    id: 'quarter',
    name: locale('季', 'datetype_unit_quarter')
  }, {
    type: 'year',
    id: 'year',
    name: locale('年', 'datetype_unit_year')
  }] : _ref$list,
      value = _ref.value,
      _ref$contentProps = _ref.contentProps,
      contentProps = _ref$contentProps === void 0 ? {} : _ref$contentProps,
      _ref$TabsProps = _ref.TabsProps,
      TabsProps = _ref$TabsProps === void 0 ? {} : _ref$TabsProps,
      _ref$DatePickerComboP = _ref.DatePickerComboProps,
      DatePickerComboProps = _ref$DatePickerComboP === void 0 ? {} : _ref$DatePickerComboP,
      onError = _ref.onError,
      onChange = _ref.onChange,
      props = _objectWithoutProperties(_ref, _excluded);

  var rootRef = useRef(null);
  var instance = useRef(null);
  useImperativeHandle(ref, function () {
    return {
      rootDOM: rootRef === null || rootRef === void 0 ? void 0 : rootRef.current,
      instance: instance.current,
      getRootDOM: function getRootDOM() {
        return rootRef === null || rootRef === void 0 ? void 0 : rootRef.current;
      },
      getInstance: function getInstance() {
        return instance.current;
      }
    };
  }); // 实例化

  instance.current = {
    getDisplayValue: function getDisplayValue(value) {
      return DateComboUtils.getDisplayValue({
        type: value === null || value === void 0 ? void 0 : value.type,
        value: value === null || value === void 0 ? void 0 : value.value
      });
    }
  }; // 显示值

  var displayValue = instance.current.getDisplayValue(value);
  useEffect(function () {
    // 如果默认没有值, 则默认为当天
    if (!value) {
      var newValue = list[0];
      updateValue(newValue);
      handleChange(newValue);
    }
  }, []); // eslint-disable-line
  // 统一的修改方法

  function handleChange(newValue, index) {
    var date = DateModalUtils.validateDate(newValue.value, {
      type: newValue.type,
      min: min,
      max: max,
      onError: onError
    });

    if (date === false) {
      return;
    }

    newValue.value = date;
    if (onChange) onChange(newValue, index);
  } // 点击Tab


  function handleTabs(newValue) {
    if (!newValue.type) return;
    newValue.value = value === null || value === void 0 ? void 0 : value.value;
    handleChange(newValue);
  } // 向前


  function handlePrev(e) {
    if (!value) return;
    var newValue = updateValue(value, -1);
    handleChange(newValue);
  } // 向后


  function handleNext(e) {
    if (!value) return;
    var newValue = updateValue(value, 1);
    handleChange(newValue);
  } // 选择日期


  function handleDate(date) {
    value.value = date;
    handleChange(value);
  }
  /**
   * 切换日期
   * @param {Date} newValue 日期
   * @param {Number} go 前行后退, 0: 当前; -1: 后退; 1: 前进;
   */


  function updateValue(newValue) {
    var _newValue, _newValue2;

    var go = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
    if (!((_newValue = newValue) !== null && _newValue !== void 0 && _newValue.type)) return;

    if (!((_newValue2 = newValue) !== null && _newValue2 !== void 0 && _newValue2.value)) {
      newValue.value = new Date();
    }

    newValue = Object.clone(newValue);

    switch (newValue.type) {
      // 年
      case 'year':
        {
          if (go === 0) {
            break;
          }

          go === -1 ? newValue.value.prevYear() : newValue.value.nextYear();
          break;
        }
      // 季

      case 'quarter':
        {
          if (go === 0) {
            break;
          }

          go === -1 ? newValue.value.prevQuarter() : newValue.value.nextQuarter();
          break;
        }
      // 月

      case 'month':
        {
          if (go === 0) {
            break;
          }

          go === -1 ? newValue.value.prevMonth() : newValue.value.nextMonth();
          break;
        }
      // 日

      case 'date':
        {
          if (go === 0) {
            break;
          }

          go === -1 ? newValue.value.prevDate() : newValue.value.nextDate();
          break;
        }

      default:
        {}
    }

    return newValue;
  }

  return /*#__PURE__*/React.createElement("div", _extends({
    ref: rootRef
  }, props, {
    className: "datepicker-types".concat(props.className ? ' ' + props.className : '')
  }), Array.isArray(list) && /*#__PURE__*/React.createElement(Tabs, _extends({
    onChange: handleTabs,
    list: list,
    value: value,
    className: "tabs-rect datepicker-types-tabs"
  }, TabsProps)), /*#__PURE__*/React.createElement("div", _extends({}, contentProps, {
    className: "datepicker-types-content".concat(contentProps.className ? ' ' + contentProps.className : '')
  }), /*#__PURE__*/React.createElement("i", {
    className: "datepicker-types-prev icon shape-arrow-left sm",
    onClick: handlePrev
  }), /*#__PURE__*/React.createElement(InputDate, _extends({}, DatePickerComboProps, {
    value: value === null || value === void 0 ? void 0 : value.value,
    type: value === null || value === void 0 ? void 0 : value.type,
    className: "datepicker-types-date".concat(DatePickerComboProps.className ? ' ' + DatePickerComboProps.className : ''),
    onChange: handleDate
  }), /*#__PURE__*/React.createElement("p", null, displayValue || '')), /*#__PURE__*/React.createElement("i", {
    className: "datepicker-types-next icon shape-arrow-right sm",
    onClick: handleNext
  })));
});
export default Types;